# @author Natapol Pornputtapong <natapol@chalmers.se>
#
require 'json'

module Sylfy
	
	module Service
	
		module ENSEMBL
			module REST
				module Alignment
					
					SERVICENAME = 'alignment'
					
					module_function
					
					# Retrieves genomic alignments as separate blocks based on its location
					#
					# @param species [String] Registry name/aliases used to restrict searches by
					#
					# @param region [String] The region to retrieve sequence for. A maximum of
					# 10Mb is allowed to be requested at any one time
					#
					# @param option [String] option of service please consult {http://beta.rest.ensembl.org/documentation/info/xref_id}
					#	
					# @return [array of Hash] results
					#
					def block(species, region, option = {})
						option_type = [:callback, :compact, :compara, :display_species_set, :mask, :method, :species_set, :species_set_group]
						get_path = "/#{SERVICENAME}/block/region/#{species}/#{region}"
						return Sylfy::Service::ENSEMBLREST::service(get_path, option_type, option)
					end
					
					# Retrieves genomic alignments as a single slice based on its location
					#
					# @param species [String] Registry name/aliases used to restrict searches by
					#
					# @param region [String] The region to retrieve sequence for. A maximum of
					# 10Mb is allowed to be requested at any one time
					#
					# @param option [String] option of service please consult {http://beta.rest.ensembl.org/documentation/info/xref_id}
					#	
					# @return [array of Hash] results
					#
					def slice(species, region, option = {})
						option_type = [:callback, :compact, :compara, :display_species_set, :expanded, :mask, :method, :overlap, :species_set, :species_set_group]
						get_path = "/#{SERVICENAME}/slice/region/#{species}/#{region}"
						return Sylfy::Service::ENSEMBLREST::service(get_path, option_type, option)
					end
					
				end
				
				module Genetree
					
					SERVICENAME = 'genetree'
					
					module_function
	
					# Retrieves Gene Tree dumps for a given Gene Tree stable identifier
					#
					# @param id [String] Ensembl GeneTree ID
					#
					# @param option [String] option of service please consult {http://beta.rest.ensembl.org/documentation/info/xref_id}
					#	
					# @return [array of Hash] results
					#
					def id(id, option = {})
						option_type = [:aligned, :nh_format , :sequence]
						get_path = "/#{SERVICENAME}/id/#{id}"
						option['Content-Type'] = 'text/x-nh'
						return Sylfy::Service::ENSEMBLREST::service(get_path, option_type, option)
					end
					
					# Retrieves the Gene Tree that contains the given stable identifier
					#
					# @param id [String] Ensembl gene ID
					#
					# @param option [String] option of service please consult {http://beta.rest.ensembl.org/documentation/info/xref_id}
					#	
					# @return [array of Hash] results
					#
					def member(id, option = {})
						option_type = [:db_type, :object_type , :species]
						get_path = "/#{SERVICENAME}/member/id/#{id}"
						option['Content-Type'] = 'text/x-phyloxml+xml'
						return Sylfy::Service::ENSEMBLREST::service(get_path, option_type, option)
					end
					
					# Retrieves a Gene Tree containing the Gene identified by the given symbol
					#
					# @param species [String] Registry name/aliases used to restrict searches by
					#
					# @param symbol [String] The external name/symbol to search for
					#
					# @param option [String] option of service please consult {http://beta.rest.ensembl.org/documentation/info/xref_id}
					#	
					# @return [array of Hash] results
					#
					def symbol(species, symbol, option = {})
						option_type = [:db_type, :external_db, :object_type]
						get_path = "/#{SERVICENAME}/member/symbol/#{species}/#{symbol}"
						option['Content-Type'] = 'text/x-phyloxml+xml'
						return Sylfy::Service::ENSEMBLREST::service(get_path, option_type, option)
					end
				end
				
				module Homology
					
					SERVICENAME = 'homology'
					
					module_function
	
					# Retrieves homology information by ensembl gene id
					#
					# @param id [String] Ensembl GeneTree ID
					#
					# @param option [String] option of service please consult {http://beta.rest.ensembl.org/documentation/info/xref_id}
					#	
					# @return [array of Hash] results
					#
					def id(id, option = {})
						option_type = [:aligned, :callback, :format, :sequence, :species,  :target_species, :target_taxon, :type]
						get_path = "/#{SERVICENAME}/id/#{id}"
						return Sylfy::Service::ENSEMBLREST::service(get_path, option_type, option)
					end
					
					# Retrieves homology information by symbol
					#
					# @param species [String] Registry name/aliases used to restrict searches by
					#
					# @param symbol [String] The external name/symbol to search for
					#
					# @param option [String] option of service please consult {http://beta.rest.ensembl.org/documentation/info/xref_id}
					#	
					# @return [array of Hash] results
					#
					def symbol(species, symbol, option = {})
						option_type = [:aligned, :callback, :compara, :external_db, :format, :sequence, :target_species, :target_taxon, :type]
						get_path = "/#{SERVICENAME}/symbol/#{species}/#{symbol}"
						return Sylfy::Service::ENSEMBLREST::service(get_path, option_type, option)
					end
				end
			end
		end
    end
end
