# 
# UniSysDB library in Ruby
# Copyright (C) 2012
#
# @author Natapol Pornputtapong <natapol@chalmers.se>
#
require 'open-uri'
require 'net/http'

module Sylfy
	
	module Service
		
		
		module LipidMap
			module REST
				@@baseuri = "http://www.lipidmaps.org/data"
				
				module_function
				# Method for setting inchi data member
				#
				# == Parameters:
				# id::
				#   LipidMap id to search
				# outputtype::
				#   TSV, CSV, SDF or MDLMOL [defualt: TSV]
				
				def lmsdRecord(id, outputtype = "TSV")
				  
					uri = "#{@@baseuri}/LMSDRecord.php?LMID=#{id}"
					uri += "&Mode=File"
					uri += "&OutputType=#{["TSV", "CSV", "SDF"].include?(outputtype) ? outputtype : 'TSV'}"
					uri += "&OutputDelimiter=Comma&OutputQuote=Yes&OutputColumnHeader=Yes"
					
					begin
						doc = URI.parse(uri).read().strip()
						case outputtype
						when 'TSV', nil
							return CSV.parse(doc, :headers => true, :col_sep => "\t")
						when 'CSV'
							return CSV.parse(doc, :headers => true)
						else
							return doc
						end
					rescue OpenURI::HTTPError
						return nil
					end
					
				end
				
				# Method for setting inchi data member
				#
				# == Parameters:
				# str::
				#   search string
				# mode::
				#   Search mode ProcessStrSearch, ProcessTextSearch or ProcessTextOntologySearch
				# outputtype::
				#   TSV, CSV, SDF or MDLMOL [defualt: TSV]
				
				def lmsdSearch(str, mode, outputtype = "TSV")
				  
					uri = "#{@@baseuri}/structure/LMSDSearch.php?#{str}&OutputMode=File&Mode=#{mode}"
					uri += "&OutputType=#{["TSV", "CSV", "SDF"].include?(outputtype) ? outputtype : 'TSV'}"
					uri += "&OutputDelimiter=Comma&OutputQuote=Yes&OutputColumnHeader=Yes"
					
					begin
						doc = URI.parse(uri).read().strip()
						case outputtype
						when 'TSV', nil
							return CSV.parse(doc, :headers => true, :col_sep => "\t")
						when 'CSV'
							return CSV.parse(doc, :headers => true)
						else
							return doc
						end
					rescue OpenURI::HTTPError
						return nil
					end
					
				end
				
				private :lmsdSearch
				
				# Method for setting inchi data member
				#
				# == Parameters:
				# string::
				#   search string
				# field::
				#   field type: LMID, Name, Formula, SMILESString [defualt: LMID]
				# exactMatch::
				#   Boolean [defualt: yes]
				# outputtype::
				#   TSV, CSV, SDF or MDLMOL [defualt: TSV]
				
				def lmsdStrSearch(string, field = 'LMID', exactMatch = true, outputtype = "TSV")
				  
					searchType = exactMatch ? 'ExactMatch' : 'SubStructure'
					
					if ['LMID', 'Name', 'Formula', 'SMILESString'].include?(field)
						return lmsdSearch("#{field}=#{URI.encode(string)}&SearchType=#{searchType}", "ProcessStrSearch", outputtype)
					else
						raise Sylfy::Service::ParameterError, "#{field} field does not in service."
					end
				end
				
				# Method for setting inchi data member
				#
				# == Parameters:
				# hash::
				#   searh hash
				#   {
				#     :LMID	=> LM_ID,
				#     :Name	=> part of any common name,
				#     :Formula => formula,
				#     :ExactMass => positive number,
				#     :ExactMassOffSet => positive number,
				#     :CoreClass => integer [1 to 8],
				#       1=Fatty Acyls [FA] 
				#       2=Glycerolipids [GL] 
				#       3=Glycerophospholipids [GP] 
				#       4=Sphingolipids [SP] 
				#       5=Sterol Lipids [ST] 
				#       6=Prenol Lipids [PR] 
				#       7=Saccharolipids [SL] 
				#       8=Polyketides [PK]
				#     :MainClass => integer,
				#       e.g.  FA10=101
				#             FA11=111
				#             ST05=505
				#     :SubClass => integer
				#       e.g.  ST0402=50402
				#   }
				#   
				# outputtype::
				#   TSV, CSV, SDF or MDLMOL [defualt: TSV]
				
				def lmsdTxtSearch(hash, outputtype = "TSV")
				  
					string = ''
					
					hash.each_pair do |k, v|
						string += '&' if string != ''
						string += "#{k.to_s}=#{v}"
					end
					
					return lmsdSearch(string, "ProcessTextSearch", outputtype)
				end
				
				# Method for setting inchi data member
				#
				# == Parameters:
				# onto::
				#   string = "<ontoStr>[,<ontoStr>,<ontoStr>,<ontoStr>]"
				#   <ontoStr> = <ParamName> <ParamModifier> 
				#   <ParamName> = carbons, doublebonds, triplebonds, rings, OH, NH2, OOH, ketones, epoxides, COOH, methyls, SH, Br, Cl, F, Methylenes, CHO, OMe, OAc, COOMe, Ester, Ether
				#   <ParamModifier> = eq, ge, le
				#   <ParamValue> = integer
				#   example: "carbons eq 20, triplebonds ge 2" or "carbons eq 10"
				# option::
				#   {
				#     :OutputType => TSV, CSV, SDF [defualt: TSV],
				#   }
				def lmsdOntoSearch(onto, outputtype = "TSV")
				  
					paramName = ["carbons", "doublebonds", "triplebonds", "rings", "OH", "NH2", "OOH", "ketones", "epoxides"]
					paramName = paramName + ["COOH", "methyls", "SH", "Br", "Cl", "F", "Methylenes", "CHO", "OMe", "OAc", "COOMe", "Ester", "Ether"]
					
					paramModifier = ["eq", "ge", "le"]
					
					i = 1
					ontostr = ""
					
					onto.chomp.gsub(/, /, ',').split(/,/).each do |str|
						dat = str.split(/ /)
						if paramName.include?(dat[0]) && paramModifier.include?(dat[1]) && i < 5
							ontostr += '&' if i > 1
							ontostr += "OntologyParamName#{i}=#{dat[0]}&OntologyParamModifier#{i}=#{dat[1]}&OntologyParamValue#{i}=#{dat[2].to_i}"
							i += 1
						end
					end
					
					if ontostr != ''
						return lmsdSearch(ontostr, "ProcessTextOntologySearch", outputtype)
					else
						raise Sylfy::Service::ParameterError, "ontology string error."
					end
				  
				  
				end
				
				
				
			end
			
			module_function
			
			# Method for setting inchi data member
			#
			# == Parameters:
			# id::
			#   LipidMap id to search
			# :id, :dataPrimarySource, :xrefs, :relations :inchi, :formula, :smiles, :inchiKey :names
			def idbyinchi(inchi)
				result = []
				if inchi =~ Sylfy::Pattern::INCHI
					smile = Rubabel[inchi, :inchi].to_s(:smi)
					REST::lmsdStrSearch(smile, 'SMILESString').each {|e| result.push(e["LM_ID"])} if smile != nil
				else
					raise Sylfy::Service::ParameterError, "#{inchi} is not InChI."
				end
				return result
			end
		end
	end
end

#test = File.open('tmp.mol', 'w')
#test.write(Unisys::Service::RESTLipidMap.lmsdRecord('LMFA08040013', {:OutputType => 'MDLMOL'}))
#test.close
#output = `obabel -imol tmp.mol -oinchi`
#p output
#Unisys::Service::RESTLipidMap.getSmallMolecule('LMFA08040013')
#puts Unisys::Service::RESTLipidMap.lmsdRecord('LMFA08040013', {:OutputType => 'MDLMOL'})
#puts Unisys::Service::RESTLipidMap.lmsdSearchStr('LMSL02000001')
#puts Unisys::Service::RESTLipidMap.lmsdSearchTxt({:CoreClass => 1, :MainClass => 101, :SubClass => 10101})
#puts Unisys::Service::RESTLipidMap.lmsdSearchOnto('carbons eq 20')
