# 
# UniSysDB library in Ruby
# Copyright (C) 2012
#
# @author Natapol Pornputtapong <natapol@chalmers.se>
#
require 'open-uri'
require 'net/http'
require 'json'

module Sylfy
	
	module Service
		
		
		module PubChem
			
			module REST
				@@baseuri = "http://pubchem.ncbi.nlm.nih.gov/rest/pug"
				@@output_type = [:XML, :ASNT, :ASNB, :JSON, :JSONP, :SDF, :CSV, :PNG, :TXT]
				@@main_oper = [:record, :aids, :sids, :cids]
				@@cmp_property = ["MolecularFormula", "MolecularWeight", "CanonicalSMILES", "IsomericSMILES", "InChI", "InChIKey", "IUPACName", "XLogP", "ExactMass", \
							"MonoisotopicMass", "TPSA", "Complexity", "Charge", "HBondDonorCount", "HBondAcceptorCount", "RotatableBondCount", "HeavyAtomCount", \
							"IsotopeAtomCount", "AtomStereoCount", "DefinedAtomStereoCount", "UndefinedAtomStereoCount", "BondStereoCount", "DefinedBondStereoCount", \
							"UndefinedBondStereoCount", "CovalentUnitCount", "Volume3D", "XStericQuadrupole3D", "YStericQuadrupole3D", "ZStericQuadrupole3D", "FeatureCount3D", \
							"FeatureAcceptorCount3D", "FeatureDonorCount3D", "FeatureAnionCount3D", "FeatureCationCount3D", "FeatureRingCount3D", "FeatureHydrophobeCount3D", \
							"ConformerModelRMSD3D", "EffectiveRotorCount3D", "ConformerCount3D"]
				@@xref_ns = ['RegistryID', 'RN', 'PubMedID', 'MMDBID', 'ProteinGI', 'NucleotideGI', 'TaxonomyID', 'MIMID', 'GeneID', 'ProbeID', 'PatentID']
				@@xref_oper = ['RegistryID', 'RN', 'PubMedID', 'MMDBID', 'DBURL', 'SBURL', 'ProteinGI', 'NucleotideGI', 'TaxonomyID', 'MIMID', 'GeneID', 'ProbeID', 'PatentID']
	
				@@assay_type = ['all', 'confirmatory', 'doseresponse', 'onhold', 'panel', 'rnai', 'screening', 'summary']
				
				#http://pubchem.ncbi.nlm.nih.gov/rest/pug/<input specification>/<operation specification>/[<output specification>][?<operation_options>]
				
				#<input specification> = <domain>/<namespace>/<identifiers>
				#	<domain> = substance | compound | assay | <other inputs>
				#		compound domain <namespace> = cid | name | smiles | inchi | sdf | inchikey | <structure search> | <xref> | listkey
				#			<structure search> = {substructure | superstructure | similarity | identity}/{smiles | inchi | sdf | cid}
				#			<xref> = xref/{RegistryID | RN | PubMedID | MMDBID | ProteinGI | NucleotideGI | TaxonomyID | MIMID | GeneID | ProbeID | PatentID}
				#		substance domain <namespace> = sid | sourceid/<source name> | sourceall/<source name> | name | <xref> | listkey
				#			<source name> = any valid PubChem depositor name
				#		assay domain <namespace> = aid | listkey | type/<assay type> | sourceall/<source name>
				#			<assay type> = all | confirmatory | doseresponse | onhold | panel | rnai | screening | summary
				#		<other inputs> = sources/[substance, assay]
				#	<identifiers> = comma-separated list of positive integers (cid, sid, aid) or identifier strings (source, inchikey, listkey); single identifier string (name, smiles, xref; inchi, sdf by POST only)
				#	
				#
				#compound domain <operation specification> = record | <compound property> | synonyms | sids | cids | aids | assaysummary | classification
				#	<compound property> = property / [comma-separated list of property tags]
				#substance domain <operation specification> = record | synonyms | sids | cids | aids | assaysummary | classification
				#assay domain <operation specification> = record | aids | sids | cids | description | targets/<target type> | <doseresponse> | summary
				#	target_type = {ProteinGI, ProteinName, GeneID, GeneSymbol}
				#	<doseresponse> = doseresponse/sid
				#
				#
				#<output specification> = XML | ASNT | ASNB | JSON | JSONP [ ?callback=<callback name> ] | SDF | CSV | PNG | TXT
				
				module_function
				
				def perform(input, operation = '', output = :JSON, option = nil)
					
					option_text = []
					option.each_pair{|k, v| option_text.push("#{k.to_s}=#{v.to_s}")} if option.class == Hash
					
					if @@output_type.include?(output.to_sym)
						uri = "#{@@baseuri}/#{URI.escape(input, "[] ")}"
						uri += "/#{operation}" if operation != nil && operation != ''
						uri += "/#{output}"
						uri += "?#{option_text.join(',')}" if option_text.length > 0
						
						case output.to_sym
						when :JSON
							JSON.parse(URI.parse(uri).read().strip())
						else
							return URI.parse(uri).read().strip()
						end
						
					else
						raise Sylfy::Service::ParameterError, "Wrong output type. Should be #{@@output_type.inspect}"
					end
					
				end
				
				# Method for setting inchi data member
				#
				# == Parameters:
				# id::
				#   LipidMap id to search
				# :id, :dataPrimarySource, :xrefs, :relations :inchi, :formula, :smiles, :inchiKey :names
				def substance(identifiers, namespace, operation = '', output = :JSON, option = nil)
					
					oper = @@main_oper + [:synonyms, :assaysummary, :classification]
					ns = [:sid, :name, :listkey]
					
					tmp_operation = ""
					
					if ns.include?(namespace.to_sym)
						namespace = namespace.to_s
					else
						namespace = namespace.to_s.split(/\//)
						if ['sourceid', 'sourceall'].include?(namespace[0])
							namespace = namespace.join('/')
						elsif namespace[0] == 'xref' && @@xref_ns.include?(namespace[1])
							namespace = namespace[0..2].join('/')
						else
							raise ServiceException::ParameterError, "Wrong domain type. Should be #{ns.join(', ')}, \
								xref/{#{@@xref_ns.join(' | ')}}, \
								sourceid/<source name> or sourceall/<source name>"
						end
					end
					
					if oper.include?(operation.to_sym)
						tmp_operation = operation.to_s
					else
						tmp_operation = ""
					end
					
					
					
					begin
						return perform("substance/#{namespace}/#{identifiers.to_s}", tmp_operation, output, option)
					rescue OpenURI::HTTPError
						return {}
					end
					
				end
				
				# Method for setting inchi data member
				#
				# == Parameters:
				# id::
				#   LipidMap id to search
				# :id, :dataPrimarySource, :xrefs, :relations :inchi, :formula, :smiles, :inchiKey :names
				def compound(identifiers, namespace, operation = '', output = :JSON, option = nil)
					oper = @@main_oper + [:synonyms, :assaysummary, :classification]
					ns = [:cid, :name, :smiles, :inchi, :sdf, :inchikey, :listkey]
					
					tmp_operation = ""
					
					if ns.include?(namespace.to_sym)
						namespace = namespace.to_s
					else
						namespace = namespace.to_s.split(/\//)
						if ['substructure', 'superstructure', 'similarity', 'identity'].include?(namespace[0]) && ['smiles', 'inchi', 'sdf', 'cid'].include?(namespace[1])
							namespace = namespace[0..1].join('/')
						elsif namespace[0] == 'xref' && @@xref_ns.include?(namespace[1])
							namespace = namespace[0..2].join('/')
						else
							raise ServiceException::ParameterError, "Wrong domain type. Should be #{ns.join(', ')}, \
								xref/{#{@@xref_ns.join(' | ')}} or \
								{substructure | superstructure | similarity | identity}/{smiles | inchi | sdf | cid}"
						end
					end
					
					if oper.include?(operation.to_sym)
						tmp_operation = operation.to_s
					else
						operation = operation.to_s.split(/\//)
						if operation[0] == 'property' #&& @@cmp_property.repeated_permutation(n).include?()
							tmp_operation = operation[0..1].join('/')
						elsif operation[0] == 'xrefs' && @@xref_oper.include?(operation[1])
							tmp_operation = operation[0..1].join('/')
						else
							tmp_operation = ""
						end
					end
					
					begin
						return perform("compound/#{namespace}/#{identifiers.to_s}", tmp_operation, output, option)
					rescue OpenURI::HTTPError
						return {}
					end
					
				end
				
				# Method for setting inchi data member
				#
				# == Parameters:
				# id::
				#   LipidMap id to search
				# :id, :dataPrimarySource, :xrefs, :relations :inchi, :formula, :smiles, :inchiKey :names
				def assay(identifiers, namespace, operation = '', output = :JSON, option = nil)
					oper = @@main_oper + [:description]
					ns = [:aid, :listkey]
					
					tmp_operation = ""
	
					if ns.include?(namespace.to_sym)
						namespace = namespace.to_s
					else
						namespace = namespace.split(/\//)
						if namespace[0] == 'type' && @@assay_type.include?(namespace[1])
							namespace = namespace[0..1].join('/')
						elsif namespace[0] == 'sourceall'
							namespace = namespace[0..1].join('/')
						else
							raise ServiceException::ParameterError, "Wrong domain type. Should be #{ns.join(', ')}, \
								type/{#{@@assay_type.join(' | ')}} or sourceall/<source name>"
						end
					end
					
					if oper.include?(operation.to_sym)
						tmp_operation = operation.to_s
					elsif operation.to_s =~ /^targets\/([\w,]+)/ #target_type = {ProteinGI, ProteinName, GeneID, GeneSymbol}
						tmp_operation = operation.to_s
					elsif operation.to_s =~ /^doseresponse\/([\w,]+)/ #doseresponse/sid
						tmp_operation = operation.to_s
					else
						tmp_operation = ""
					end
					
					begin
						return perform("assay/#{namespace}/#{identifiers.to_s}", tmp_operation, output, option)
					rescue OpenURI::HTTPError
						return {}
					end
					
				end
				
				# Method for setting inchi data member
				#
				# == Parameters:
				# id::
				#   LipidMap id to search
				# :id, :dataPrimarySource, :xrefs, :relations :inchi, :formula, :smiles, :inchiKey :names
				def idconv(id)
					return JSON.load(compound(id, 'xref/RegistryID', "cids"))["IdentifierList"]["CID"]
				end
				
				
			end
			
		end
	end
end



#require 'json'
#require './unisys.rb'

#http://pubchem.ncbi.nlm.nih.gov/rest/pug/assay/aid/1000/sids/XML?sids_type=inactive

#p JSON.load(Unisys::Service::RESTPubChem.assay("1000", :aid, :sids, :JSON ,{:sids_type => :inactive}))
#p JSON.load(Unisys::Service::RESTPubChem.substance("747285", "sourceid/DTP.NCI"))
#p JSON.load(Unisys::Service::RESTPubChem.compound("2244", :cid, "property/MolecularFormula,CanonicalSMILES,InChI,InChIKey,IUPACName", "JSON"))
#p Unisys::Service::RESTPubChem.getSmallMolecule(2244, :cid)
#p Unisys::Service::RESTPubChem.idconv('D00001')
#p JSON.load(Unisys::Service::RESTPubChem.compound("LMFA08040013", 'xref/RegistryID', "property/MolecularFormula,CanonicalSMILES,InChI,InChIKey,IUPACName", "JSON"))
